#!/usr/bin/python3

# Test that the Toolbar works correctly.

from testCommon import run_app, bail

from dogtail.procedural import *

try:
    run_app(file='LoremIpsum.txt')

    # NEW 
    click('New', roleName='push button')
    focus.widget('LoremIpsum.txt', roleName='page tab')
    focus.widget.node.select()

    # OPEN
    click('Open', roleName='push button')
    click('Cancel', roleName='push button')
 
      
    focus.application('xed')
    click('Edit', roleName='menu')
    click('Select All', roleName='menu item')
    focus.application('xed')
    type('testing')

    # UNDO
    click('Undo', roleName='push button')
    # REDO
    click('Redo', roleName='push button')

    # THE ISSUE OF NEEDING TO PRESS UNDO TWICE AS BEEN DOCUMENTED AS A 
    # FEATURE REQUEST (#344)
    # ONCE THAT FR HAS BEEN IMPLEMENTED, ONE OF THESE LINES CAN BE REMOVED
    click('Undo', roleName='push button')
    click('Undo', roleName='push button')

    # CUT
    keyCombo('<Ctrl>a')
    click('Cut', roleName='push button')

    # PASTE
    click('Paste', roleName='push button')

    # COPY
    keyCombo('<Ctrl>a')
    click('Copy', roleName='push button')
    click('New', roleName='push button')
    click('Paste', roleName='push button')

    # SEARCH
    click('Find', roleName='push button')
    type('search test')

    # REPLACE
    click('Find and Replace', roleName='push button')
    type('replace test')

    # QUIT APPLICATION
    focus.application('xed')
    click('File', roleName='menu')
    click('Quit', roleName='menu item')
    click('Close without Saving', roleName='push button')

except:
    bail()



#!/usr/bin/python3

import os
import sys
import signal

os.environ['LANG'] = 'C'

from dogtail.config import config
config.logDebugToStdOut = True
config.logDebugToFile = False

import dogtail.procedural as dt

def run_app(file=None):
    global pid

    if file is not None:
        arguments = os.path.join(os.path.dirname(__file__), file)
    else:
        arguments = ''
    pid = dt.run(sys.argv[1], arguments=arguments, appName='xed')

def bail():
    os.kill(pid, signal.SIGTERM)
    sys.exit(1)



# Copyright (C) 2007-2008 Martin Szulecki
# Copyright (C) 2011 Jean-Philippe Fleury <contact@jpfleury.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''
Adds context menu item to open an URI at the pointer position
'''

from gi.repository import Gtk, Xed, Gio, GObject, GtkSource
import gettext
import re
import sys
import os
import subprocess
import string

gettext.install("xed")

ACCEPTED_SCHEMES = ['file', 'ftp', 'sftp', 'smb', 'dav', 'davs', 'ssh', 'http', 'https']
RE_DELIM = re.compile(r'[\w#/\?:%@&\=\+\.\\~-]+', re.UNICODE|re.MULTILINE)
RE_URI_RFC2396 = re.compile(r"((([a-zA-Z][0-9a-zA-Z+\-\.]*):)?/{0,2}([0-9a-zA-Z;:,/\?@&=\+\$\.\-_!~\*'\(\)%]+))?(#[0-9a-zA-Z;,/\?:@&\=+$\.\\-_!~\*'\(\)%]+)?")

class OpenURIContextMenuPlugin(GObject.Object, Xed.WindowActivatable):
	__gtype_name__ = "OpenURIContextMenuPlugin"
	window = GObject.property(type=Xed.Window)

	def __init__(self):
		GObject.Object.__init__(self)
		self.uri = ""
		self.window = None
		self.encoding = GtkSource.Encoding.get_from_charset("UTF-8")

	def do_activate(self):
		handler_ids = []
		for signal in ('tab-added', 'tab-removed'):
			method = getattr(self, 'on_window_' + signal.replace('-', '_'))
			handler_ids.append(self.window.connect(signal, method))
		self.window.OpenURIContextMenuPluginID = handler_ids
		for view in self.window.get_views():
			self.connect_view(view)

	def do_deactivate(self):
		widgets = [self.window] + self.window.get_views()
		for widget in widgets:
			handler_ids = widget.OpenURIContextMenuPluginID
			if not handler_ids is None:
				for handler_id in handler_ids:
					widget.disconnect(handler_id)
			widget.OpenURIContextMenuPluginID = None
		self.window = None

	def connect_view(self, view):
		handler_id = view.connect('populate-popup', self.on_view_populate_popup)
		view.OpenURIContextMenuPluginID = [handler_id]

	def update_ui(self, window):
		pass

	def browse_url(self, menu_item, url):
		command = ['xdg-open', url]
		# Avoid to run the browser as user root
		if os.getuid() == 0 and os.environ.has_key('SUDO_USER'):
			command = ['sudo', '-u', os.environ['SUDO_USER']] + command
		subprocess.Popen(command)

	def on_window_tab_added(self, window, tab):
		self.connect_view(tab.get_view())

	def on_window_tab_removed(self, window, tab):
		pass

	def on_view_populate_popup(self, view, menu):
		doc = view.get_buffer()
		win = view.get_window(Gtk.TextWindowType.TEXT);
		ptr_window, x, y, mod = win.get_pointer()
		x, y = view.window_to_buffer_coords(Gtk.TextWindowType.TEXT, x, y);

		# First try at pointer location
		# Starting with GTK 3.20, get_iter_at_location returns a tuple of type
		# (gboolean, GtkTextIter).  Earlier versions return only the GtkTextIter.
		insert = view.get_iter_at_location(x, y);
		if isinstance(insert, tuple):
			insert = insert[1] if insert[0] else None

		# Second try at cursor
		if insert == None:
			insert = doc.get_iter_at_mark(doc.get_insert())
			if isinstance(insert, tuple):
				insert = insert[1] if insert[0] else None

		while insert.forward_char():
			if not RE_DELIM.match(insert.get_char()):
				break

		start = insert.copy()
		while start.backward_char():
			if not RE_DELIM.match(start.get_char()):
				start.forward_char();
				break

		word = doc.get_text(start, insert, False)
		if len(word) == 0:
			return True

		word = self.validate_uri(word)
		if not word:
			return True

		displayed_word = word
		if len(displayed_word) > 50:
			displayed_word = displayed_word[:50] + u"\u2026"

		browse_to = False
		if word.startswith("http://") or word.startswith("https://"):
			browse_to = True

		if browse_to:
			browse_uri_item = Gtk.MenuItem(_("Open '%s'") % (displayed_word))
			browse_uri_item.connect('activate', self.browse_url, word);
			browse_uri_item.show();

			separator = Gtk.SeparatorMenuItem()
			separator.show();
			menu.prepend(separator)
			menu.prepend(browse_uri_item)
		return True

	def validate_uri(self, uri):
		m = RE_URI_RFC2396.search(uri);
		if not m:
			return False

		target = m.group()

		if m.group(4) == None or m.group(4) == "/":
			return False

		if m.group(2) != None:
			if m.group(3) in ACCEPTED_SCHEMES:
				return target
			else:
				return False
		else:
			if m.group(4).startswith("www."):
				return 'http://' + target

		target = os.path.expanduser(target)
		if os.path.isfile(target):
			if os.path.isabs(target):
				return 'file://' + target

		doc_dir = self.window.get_active_document().get_uri_for_display()
		if doc_dir != None:
			if doc_dir.startswith('file://'):
				f = os.path.join(os.path.dirname(doc_dir), target)
				if os.path.isfile(f.replace('file://', '', 1)):
					return f
			else:
				return os.path.join(os.path.dirname(doc_dir), target)

		paths = string.split(os.environ["PATH"], os.pathsep)
		for dirname in paths:
			f = os.path.join(os.path.dirname(dirname), 'include', target)
			if os.path.isfile(f):
				return 'file://' + f

		return False



#!/usr/bin/python3

# This test opens the Help menu and runs through the menu items.

from testCommon import run_app, bail

from dogtail.procedural import *

try:
    run_app()

    # Contents
    click('Help', roleName='menu')
    click('Contents', roleName='menu item')
    focus.dialog('Text Editor')
    keyCombo('<Control>w')

    # Keyboard Shortcuts
    click('Help', roleName='menu')
    click('Keyboard Shortcuts', roleName='menu item')
    keyCombo('<Alt>F4')

    # About
    click('Help', roleName='menu')
    click('About', roleName='menu item')
    focus.dialog('About xed')
    click('Close', roleName='push button')

    # Quit application
    click('File', roleName='menu')
    click('Quit', roleName='menu item')

except:
    bail()



#!/usr/bin/python3

# Test that the File menu and menu items work correctly.

from testCommon import run_app, bail

from dogtail.procedural import *

try:
    run_app(file='LoremIpsum.txt')

    # Save a Copy
    click('File', roleName='menu')
    click('Save As...', roleName='menu item')
    click('Cancel', roleName='push button')

    # Print Preview
    focus.application('xed')
    click('File', roleName='menu')
    click('Print Preview', roleName='menu item')
    click('Close preview', roleName='push button')

    # Print
    focus.application('xed')
    click('File', roleName='menu')
    click('Print...', roleName='menu item')
    focus.dialog('Print')
    click('Page Setup', roleName='page tab')
    click('Text Editor', roleName='page tab')
    click('Cancel', roleName='push button')

    # Quit application
    focus.application('xed')
    click('File', roleName='menu')
    click('Quit', roleName='menu item')

except:
    bail()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# generate-plugin.py - xed plugin skeletton generator
# This file is part of xed
#
# Copyright (C) 2006 - Steve Frécinaux
#
# xed is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# xed is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with xed; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

import re
import os
import argparse
from datetime import date
import preprocessor

TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), 'plugin_template')
PLUGIN_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__), '../plugins'))

# Parsing command line options
parser = argparse.ArgumentParser()
parser.add_argument('-S', '--standalone', dest='standalone', action='store_true',
                    help='indicates that this plugin is intended to be distributed as a standalone package')
parser.add_argument('-s', '--with-side-pane', dest='side_pane', action='store_true',
                    help='Indicates that this plugin will use a side pane')
parser.add_argument('-b', '--with-bottom-pane', dest='bottom_pane', action='store_true',
                    help='Indicates that this plugin will use a bottom pane')
parser.add_argument('-m', '--with-menu', dest='menu', action='store_true',
                    help='Indicates that this plugin will use menu entries')
parser.add_argument('-c', '--with-config', dest='config', action='store_true',
                    help='Indicates that this plugin will use a configuration dialog')
parser.add_argument('-d', '--description', dest='description', default='Type here a short description of your plugin', metavar='DESC',
                    help='Description of the plugin')
parser.add_argument('-a', '--author', dest='author', default=os.getenv('USERNAME'), metavar='AUTH',
                    help='Author of the plugin')
parser.add_argument('-e', '--email', dest='email', default=os.getenv('LOGNAME') + '@email.com', metavar='EMAIL',
                    help='Email address of the author')
parser.add_argument('-l', '--language', dest='language', default='c', metavar='LANG',
                    help='Language of the plugin')
parser.add_argument('-o', '--output-directory', dest='directory', default=None, metavar='LANG',
                    help='Language of the plugin')
parser.add_argument('name', metavar='PLUGIN_NAME',
                    help='The name of the plugin')

args = parser.parse_args()

plugin_name = args.name
plugin_id = re.sub('[^a-z0-9_]', '', plugin_name.lower().replace(' ', '_'))
plugin_module = plugin_id.replace('_', '-')

directives = {
    'PLUGIN_NAME'       : plugin_name,
    'PLUGIN_MODULE'     : plugin_module,
    'PLUGIN_ID'         : plugin_id,
    'AUTHOR_FULLNAME'   : args.author,
    'AUTHOR_EMAIL'      : args.email,
    'DATE_YEAR'         : date.today().year,
    'DESCRIPTION'       : args.description,
}

# Files to be generated by the preprocessor, in the form "template : outfile"
output_files = {
    'meson.build': 'meson.build',
    'xed-plugin.desktop.in': '%s.plugin.desktop.in' % plugin_module
}

if args.language == 'c':
    directives['HAS_C_FILES'] = True
    output_files['xed-plugin.c'] = 'xed-%s-plugin.c' % plugin_module
    output_files['xed-plugin.h'] = 'xed-%s-plugin.h' % plugin_module

if args.side_pane:
    directives['WITH_SIDE_PANE'] = True

if args.bottom_pane:
    directives['WITH_BOTTOM_PANE'] = True

if args.menu:
    directives['WITH_MENU'] = True

if args.config:
    directives['WITH_CONFIGURE_DIALOG'] = True

if args.directory is None:
    directory = os.getcwd() if args.standalone else PLUGIN_DIR
elif os.path.isdir(args.directory):
    directory = args.directory
else:
    print('Unable to create plugin: %s does not exist or is not a directory' % args.directory)
    quit(1)

directory = os.path.join(directory, plugin_module)
if os.path.exists(directory):
    print('Unable to create plugin: directory %s already exists' % directory)
    quit(1)
else:
    os.makedirs(directory)

if not args.standalone:
    with open(os.path.join(PLUGIN_DIR, 'meson.build'), 'r') as f:
        contents = f.read()
        lines = contents.split('\n')

        start = False
        for i in range(len(lines)):
            line = lines[i].rstrip()
            print(line)
            if line.startswith('subdir('):
                start = True
            elif start:
                break
            else:
                continue

            if line[8:-2] > plugin_module:
                break

        lines.insert(i, 'subdir(\'%s\')' % plugin_module)

    with open(os.path.join(PLUGIN_DIR, 'meson.build'), 'w') as f:
        f.write('\n'.join(lines))

# Generate the plugin base
for infile, outfile in output_files.iteritems():
    print('Generating file %s from template %s...' % (outfile, infile))

    file_directives = directives.copy()

    infile = os.path.join(TEMPLATE_DIR, infile)
    outfile = os.path.join(directory, outfile)

    if not os.path.isfile(infile):
        print('Input file %s does not exist: skipping' % os.path.basename(infile))
        continue

    # Make sure the destination directory exists
    if not os.path.isdir(os.path.split(outfile)[0]):
        os.makedirs(os.path.split(outfile)[0])

    # Variables relative to the generated file
    file_directives['DIRNAME'], file_directives['FILENAME'] = os.path.split(outfile)

    # Generate the file
    preprocessor.process(infile, outfile, file_directives)

print('Done')

# ex:ts=4:et:
